#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<avr/pgmspace.h>
#include	<stdio.h>

#include 	"lcd.h"
#include	"cdrom.h"
#include 	"mydefs.h"

#define		PRINT_DEBUG
#define 	ATAPIBUFFERSIZE 		128

uint8_t	 	atapi_buffer[ATAPIBUFFERSIZE];	
uint8_t 	mem_toc     [ATAPIBUFFERSIZE];			// 3 byte * maxsoung
uint8_t 	tmp_buffer  [50];						// jnut define 50 engugh or maybe

extern		uint8_t 	bPlaying;					// playing
extern		uint8_t		buff_lcd[STRDSPLMAX];
extern 	uint16_t	total_song;
extern		uint8_t 	bTrayOpen;				// Tray closed

uint8_t	 	cdda;
uint16_t	track_time;
//-----------------------------------------------------------------------------//
void delay_wait (uint16_t n)
{
	for (;n > 0;n--)
		_delay_ms (500);
}
//-----------------------------------------------------------------------------//
void ATAPI_ClearBuf (void)
{
	memset (atapi_buffer, 0, sizeof atapi_buffer);
}
//-----------------------------------------------------------------------------//
void SetAddress (uint8_t addr)
{
	addr 		|= 	0xE0;
	CTRL_PORT 	= 	addr;
}
//-----------------------------------------------------------------------------//
uint8_t ReadByte (uint8_t addr)
{
	register uint8_t	tmp;
		
		SetAddress (addr);
		
		DATA_DDR	= 0;		//all input
		ADDR_DDR	= 0;		//all input
		DATA_PORT	= 0xff;		//Active pull high
		CTRL_PORT &=  ~_BV(CTRL_RD);
		_delay_us (1);
		tmp = DATA_IN;
		CTRL_PORT |= _BV(CTRL_RD);
	return tmp;
}
//-----------------------------------------------------------------------------//
void WriteByte (uint8_t addr, uint8_t dat)
{

	SetAddress (addr);
	
	DATA_DDR 	= 0xff;		// all output
	DATA_PORT	= dat;
	ADDR_PORT	= 0;

	CTRL_PORT	&= ~_BV(CTRL_WR);
	_delay_us (1);
	CTRL_PORT  |= _BV(CTRL_WR);
}
//-----------------------------------------------------------------------------//
uint8_t CheckBusy (void)
{
	uint16_t timeout = 0;
	uint8_t  nRet;
	
	while (((	ReadByte(CMD_A | CMD_STATUS) & SR_BUSY) == SR_BUSY) 
				&& (++timeout < 10000))
		_delay_ms(2); //100
		nRet = (timeout < 10000) ? 0:1;
	return nRet;
}
//-----------------------------------------------------------------------------//
void ATA_SW_Reset (void)
{
	uint8_t	i = 10;
	
		WriteByte (CMD_A | CMD_COMMAND, ATAPI_RESET);
			_delay_ms (500);
	
	while (CheckBusy () && i)	i--;
}
//-----------------------------------------------------------------------------//
uint8_t ATAPI_Command (void)
{
	uint8_t	i;
	uint8_t	*miniptr = atapi_buffer;
	
	CheckBusy ();
	
	WriteByte (CMD_A | CMD_FEATURES, 0);
	WriteByte (CMD_A | CMD_CYL_HIGH, 8);
	WriteByte (CMD_A | CMD_CYL_LOW,	 0);
	WriteByte (CMD_A | CMD_COMMAND, ATAPI_PACKET );		//Write command Package
	
		_delay_us (1);
		CheckBusy ();
		SetAddress(CMD_A | CMD_DATA);
		DATA_DDR = 0xff;
		ADDR_DDR = 0xff;								//all output
		
		i = 6;
		do {	
				DATA_PORT = *miniptr++;
				ADDR_PORT = *miniptr++;

				CTRL_PORT &= ~_BV(CTRL_WR);
				_delay_us (1);
				CTRL_PORT |= _BV(CTRL_WR);
				_delay_us (1);
			} while (--i > 0);
		
		_delay_us (2);
		ReadByte (CTRL_A | CTRL_ALT_STAT);	CheckBusy();
	return (ReadByte (CMD_A | CMD_STATUS) & SR_ERR);	//Return Command Status
}
//-----------------------------------------------------------------------------//
uint8_t ATAPI_Identify(uint8_t *Buffer)
{
	uint8_t	j;

	CheckBusy ();
	
	WriteByte (CMD_A | CMD_FEATURES, 0);
	WriteByte (CMD_A | CMD_CYL_HIGH, 8);
	WriteByte (CMD_A | CMD_CYL_LOW,	 0);
	WriteByte (CMD_A | CMD_COMMAND, ATAPI_IDENTIFY);		//Write command Package
	
	_delay_us (5);
	
	CheckBusy ();
	SetAddress(CMD_A + CMD_DATA);	// setup addressing and chip selects
		
	DATA_DDR  = 0;					// port A as input
	ADDR_DDR  = 0;					// port C as input
	_delay_us (2);
		
	DATA_PORT = 0xff;				// activate Pullups
	ADDR_PORT = 0xff;				// activate Pullups
	_delay_us (2);	
	
	uint8_t a,b;
	j = ATAPIBUFFERSIZE;
	
//	printf ("\r\n---Identify \r\n");
	do
	{
		CTRL_PORT &= ~_BV(CTRL_RD); // set DIOR lo
		_delay_ms (10);
		j-=2;						 // nop replacement, allow pin change
		a = DATA_IN;
		b = ADDR_IN;
		*Buffer++ = b;		 		// read lo byte	
		*Buffer++ = a;		 		// read hi byte
		CTRL_PORT |= _BV(CTRL_RD);
	
	//printf ("%02X,%02X ",a,b);
		
	} while (j > 0);
	
	ReadByte(CTRL_A | CTRL_ALT_STAT);	CheckBusy();
		
	return (ReadByte(CMD_A | CMD_STATUS) & SR_ERR);		// return 0 if OK, 1 if error
}
//-----------------------------------------------------------------------------//
// Check ATA data request status, 0=OK, 1=ERROR, 2=Timeout
uint8_t CheckDataReq(void)
{
	uint8_t  	stat;
	uint8_t		bState = 0;
	uint16_t 	timeout = 0;
	uint16_t 	maxtout = 500 + (50000 * (bState == 0));
	
	while(1)
	{
		stat = ReadByte(CMD_A | CMD_STATUS);
		if (stat & SR_ERR) 
		{
//#ifdef 	PRINT_DEBUG		
//			printf("\r\nDREQ ERROR");
//#endif			
			return 1;
		}
		if (++timeout > maxtout) 
		{
//#ifdef 	PRINT_DEBUG
//			printf("\r\nDREQ Timeout");
//#endif			
			return 2;
		}
		if ((stat & (SR_BUSY | SR_DRQ)) == SR_DRQ)
				return 0;
		_delay_us(5);
	}
}
//-----------------------------------------------------------------------------//
uint8_t GetDataFromCD (uint8_t *Buffer)
{
	uint8_t  i,s;
	uint8_t	 j = 0;

	for (;;)
	{
		i = 0;
	
		while(ATAPI_Command() && (++j < 5)); // Issue ATAPI Packet command
			_delay_ms((cdda) ? 15 : 3);

		if (++j > 5)
		{
			ATA_SW_Reset();
			_delay_ms(10);
			return CD_STAT_UNDEF_ERROR;	    // Too more errors > EXIT
		}
		
		if ((s = CheckDataReq()))				// If error
		{
			i = ReadByte(CMD_A | CMD_ERROR);
			if (i == 0x20)
			{
				j = CD_STAT_NODISK;				// Media not present > EXIT
				return j;
			}
			if (s == 2) _delay_ms(5);
		}
		else
			break;
	}	
	
	i = (atapi_buffer[0] == ATAPI_REQUEST_SENSE || atapi_buffer[0] == ATAPI_MODE_SENSE) ? 1 : BPS/ATAPIBUFFERSIZE;
	do										// cycle sector by atapi_buffers 
	{
		if(i-- == 0)						// Check count of sended 128 bytes blocks
			return 1;						// CD-ROM try send to more data

		SetAddress(CMD_A + CMD_DATA);		// setup addressing and chip selects
		
		DATA_DDR  = 0;						// port A as input
		ADDR_DDR  = 0;						// port C as input
		_delay_us (1);
		
		DATA_PORT = 0xff;					// activate Pullups
		ADDR_PORT = 0xff;					// activate Pullups

		uint8_t a,b;
		j = ATAPIBUFFERSIZE;

		do
		{
			CTRL_PORT &= ~_BV(CTRL_RD); // set DIOR lo
			_delay_us (2);
			j -= 2;						 // nop replacement, allow pin change
			a = DATA_IN;
			b = ADDR_IN;
			*Buffer++ = a;		 		// read lo byte	
			*Buffer++ = b;		 		// read hi byte
			CTRL_PORT |= _BV(CTRL_RD);
		} while (j > 0);

	} while ((ReadByte(CMD_A | CMD_STATUS) & (SR_BUSY | SR_DRQ)) == SR_DRQ);

	ReadByte(CTRL_A | CTRL_ALT_STAT);	CheckBusy();
	
	return (ReadByte(CMD_A | CMD_STATUS) & SR_ERR);		// return 0 if OK, 1 if error
}
//-----------------------------------------------------------------------------//
// Check CD-ROM status
uint8_t ATAPI_CheckStatus (void)
{
	uint8_t nRet;
	uint8_t cdbuff[128];
	
	ATAPI_ClearBuf();
	
	atapi_buffer[0] = ATAPI_MODE_SENSE;			// Request Mode Sense
	atapi_buffer[2] = 0x2A;						// Mechanical status page
	atapi_buffer[8] = 0x18;
	
	GetDataFromCD(cdbuff);
	
	if(cdbuff[2] == 0x71)		
			return CD_STAT_OPENED;

	ATAPI_ClearBuf ();
	ATAPI_Command ();							// Execute TEST UNIT READY command
	_delay_ms(1);

	ATAPI_ClearBuf ();
	atapi_buffer[0] = ATAPI_REQUEST_SENSE;		// Request Sense for TEST UNIT READY command
	atapi_buffer[4] = 18;						// Allocation length
	
	GetDataFromCD(cdbuff);
	
/*
#ifdef 	PRINT_DEBUG
	uint8_t index;
	printf ("\r\n---- DEBUG-----\r\n");
	for (index = 0; index  < BPS/atapi_bufferSIZE; index++)
	{
		printf ("%02X ",cdbuff[index]);
	}
	printf ("\r\n---- END DEBUG-----\r\n");
	printf ("\r\nbuf 12 = %02X",cdbuff[12]);
#endif
*/
	lcd_gotoxy (1,2);
	
	switch (cdbuff[12])
	{
		case 0x29:
		case 0x04:	nRet =  CD_STAT_NOT_READY; 	  // not Ready
					lcd_printf ("STAT NOT READY",DSPL_LEFT);
					break;
					
		case 0x3A:	nRet =  CD_STAT_NODISK;       // No disc
					lcd_printf ("STAT NOT NO Disk",DSPL_LEFT);
					break;

		case 0x00:	nRet =  CD_STAT_OK;			  // Ready
					lcd_printf ("CD STAT OK",DSPL_LEFT);
					break;
		default:	nRet =  CD_STAT_UNDEF_ERROR;  // undefine Error
					//sprintf (buff_lcd,"Error %02X",cdbuff[12]);
					//lcd_printf (buff_lcd,DSPL_LEFT);
					break;
	}
	return nRet; // nRet;
}
//-----------------------------------------------------------------------------//
// Init Audio, get TOC
uint8_t ATAPI_DA_GetToc(void)
{
	uint8_t Err;
	
		memset (mem_toc, 0, sizeof mem_toc);
		
			mem_toc[5] 		= 0xFF;				// clear audio toc marker
			ATAPI_ClearBuf();				    // clear command buffer
			atapi_buffer[0]	= 0x43;				// READ TOC command
			atapi_buffer[1]	= 0x02;				// MSF Format
			atapi_buffer[7]	= 0x08;				// 2 kB
			
			Err = GetDataFromCD(mem_toc);			// Get data from CD

			if ((mem_toc[5] & 0x0f) > 3)			// if not audio CD
				Err = CD_STAT_NOT_READY;		// return no audio CD
	return Err;
}
//-----------------------------------------------------------------------------//
// 	AUDIO CD FUNCTIONS
uint8_t calcoffset(uint8_t track, uint8_t offset)
{
	return mem_toc[(track * 8) + offset];
}
//-----------------------------------------------------------------------------//
// get M for Track #
uint8_t  ATAPI_DA_GetM(uint8_t Track)
{
	return calcoffset(Track, 9);
}
//-----------------------------------------------------------------------------//
// get S for Track #
uint8_t  ATAPI_DA_GetS(uint8_t Track)
{
	return calcoffset(Track, 10);
}
//-----------------------------------------------------------------------------//
// get F for Track #
uint8_t  ATAPI_DA_GetF(uint8_t Track)
{
	return calcoffset(Track, 11);
}
//-----------------------------------------------------------------------------//
// get total # of tracks on the CD
uint8_t ATAPI_DA_GetTrackN (void)
{
	union u16_convert num;
	
		num.bytes.high = mem_toc[0];
		num.bytes.low  = mem_toc[1];
		
	return ((num.value - 4) / 8 );
}
//-----------------------------------------------------------------------------//
uint16_t convert_to_ms (uint8_t min, uint8_t sec)
{
	return (uint16_t)min*60 + (uint16_t)sec;
}
//-----------------------------------------------------------------------------//
// get total disc time (in seconds)
uint16_t ATAPI_DA_GetDiscTime(void)
{
	return	(convert_to_ms( ATAPI_DA_GetM(ATAPI_DA_GetTrackN()), 
				            ATAPI_DA_GetS(ATAPI_DA_GetTrackN())) -2);
}
//-----------------------------------------------------------------------------//
void LCD_Disc_Time(uint8_t *des)
{
	uint8_t 	min,sec;
	uint16_t	nTime;
	uint8_t 	str[8];
	
			nTime = ATAPI_DA_GetDiscTime ();
			min = nTime / 60; sec = nTime % 60;
			sprintf (str,"%02d:%02d",min,sec);
	
	memcpy (des, str, strlen (str));
}
//-----------------------------------------------------------------------------//
uint16_t dirlist (void)
{
	return (ATAPI_DA_GetTrackN());
}
//-----------------------------------------------------------------------------//
// Load or Unload CD
void CD_Cmd(uint8_t Funct)
{
	ATAPI_ClearBuf();
		atapi_buffer[0] = ATAPI_START_STOP_UNIT;
		atapi_buffer[1] = 0x01;		// Immed = 1
		atapi_buffer[4] = Funct;
	ATAPI_Command();
}
//-----------------------------------------------------------------------------//
uint8_t LoadCD(void)				// true = ERROR
{
	uint8_t		stat,nRet = 0;
	uint8_t	 	rt1;
		
	if (bTrayOpen)
	{
		CD_Cmd(LOAD_CD);
		_delay_ms(200);
		while(ATAPI_CheckStatus() == CD_STAT_OPENED);
	}
		
	bTrayOpen 	= 0;

	_delay_ms(100);
	
	rt1 = 30;

	while(((stat = ATAPI_CheckStatus()) & 0x0F) == CD_STAT_NOT_READY)
	{
		_delay_ms(500);
		if (stat == CD_STAT_OPENED)
		{
			Do_Eject();
			return nRet;
		}
		if(--rt1 == 0)
		{
			lcd_gotoxy (1,2);
			lcd_printf ("No Disc....",DSPL_LEFT);
			return nRet;
		}	
	}
		
	if (stat != CD_STAT_NOT_READY)
	{
		//Read Name of Drive
		ATAPI_Identify(atapi_buffer);
		memset (buff_lcd, 0, sizeof buff_lcd);
		memcpy (buff_lcd, atapi_buffer+63,15);
		//print to lcd
		lcd_gotoxy (1,1);
		lcd_printf (buff_lcd,DSPL_LEFT);
		
		if (stat == CD_STAT_OK)
		{
			if (ATAPI_DA_GetToc () != 0)
			{
				lcd_gotoxy (1,2);
				lcd_printf ("Wrong Disc",DSPL_LEFT);
				CD_Cmd(STOP_CD);
				return nRet;
			}
			else
			{
				lcd_gotoxy (1,2);
				lcd_printf ("Audio CD",DSPL_LEFT);
			}
			CD_Cmd(STOP_CD);
			nRet = 1;
		}
	}
	return nRet;
}
//-----------------------------------------------------------------------------//
void Do_Eject (void)
{
	if (bTrayOpen == 0)
	{
		CD_Cmd(EJECT_CD);
		bTrayOpen = 1;
    }
	else
		LoadCD ();
}
//-----------------------------------------------------------------------------//
// Init ATA
uint8_t set_drive (uint8_t drive)
{
	uint8_t nRet = 0;
	
	WriteByte(CMD_A | CMD_DEV_HEAD, drive);		// Set Drive/Head register
	
	_delay_us(10);

	ATA_SW_Reset ();	

	if ( 	ReadByte(CMD_A | CMD_SECT_CNT) == 0x01 && 
			ReadByte(CMD_A | CMD_SECT_NUM) == 0x01 &&		
			ReadByte(CMD_A | CMD_CYL_LOW)  == 0x14  && 
			ReadByte(CMD_A | CMD_CYL_HIGH) == 0xEB)		// check ATAPI signature
				nRet = 1;
	return nRet;	
}
//-----------------------------------------------------------------------------//
uint8_t ATA_Init (void)
{
	uint8_t nRet = 0;
	
		lcd_gotoxy (1,2);

		if(set_drive(DH_CHS0))		// try Drive 0 (MASTER)
			lcd_printf ("Master Drive",DSPL_LEFT);
		else
		{
			if(set_drive(DH_CHS1))	// try Drive 1 (SLAVE)
				lcd_printf ("SALVE Drive",DSPL_LEFT);
			else
			{
				lcd_printf ("None Drive",DSPL_LEFT);
				nRet = 1;
			}	
		}		
	return nRet;
}
//-----------------------------------------------------------------------------//
void init_port (void)
{
	CTRL_DDR	= 0xff;
}
//-----------------------------------------------------------------------------//
void ATA_HW_Reset (void)
{
	init_port ();
	
	CTRL_PORT |= _BV(CTRL_RST);
	_delay_ms (100);
}
//-----------------------------------------------------------------------------//
//Audio CD Mode
//-----------------------------------------------------------------------------//
// Play From M:S:F
uint8_t  ATAPI_DA_PlayMSF(uint8_t m, uint8_t s, uint8_t f)
{
	ATAPI_ClearBuf();
		atapi_buffer[0] = 0x47;					// Command code (PLAY)
		atapi_buffer[3] = m;					// start M
		atapi_buffer[4] = s;					// start S
		atapi_buffer[5] = f;					// start F
		atapi_buffer[6] = ATAPI_DA_GetM(ATAPI_DA_GetTrackN());	// end M
		atapi_buffer[7] = ATAPI_DA_GetS(ATAPI_DA_GetTrackN());	// end S
		atapi_buffer[8] = ATAPI_DA_GetF(ATAPI_DA_GetTrackN());	// end F
	return ATAPI_Command();					// 0 if OK, else ERROR
}
//-----------------------------------------------------------------------------//
// Play FromTrack
uint8_t  ATAPI_DA_Play(uint8_t Track)
{
	return ATAPI_DA_PlayMSF(ATAPI_DA_GetM(Track), ATAPI_DA_GetS(Track), 
							ATAPI_DA_GetF(Track));
}
//-----------------------------------------------------------------------------//
// Stop Playing
uint8_t  ATAPI_DA_Stop(void)
{
		ATAPI_ClearBuf();
		atapi_buffer[0] = 0x4E;			// Command code (STOP)
	return ATAPI_Command();			// 0 if OK, else ERROR
}
//-----------------------------------------------------------------------------//
// Pause, resume (bResume == true) playing
uint8_t  ATAPI_DA_PauseResume (uint8_t bResume)
{
	ATAPI_ClearBuf();
		atapi_buffer[0] = 0x4B;			// Command code (PAUSE)
		atapi_buffer[8] = bResume;		// 0 -> pause, 1 -> resume
	return ATAPI_Command();				// 0 if OK, else ERROR
}
//-----------------------------------------------------------------------------//
// Get SubChannel Data to BUFFER1 memory (buffers is not used in CDDA mode)
uint8_t  ATAPI_DA_GetSubC (void)
{
	uint8_t err;
	
		CheckBusy();
		memset (tmp_buffer, 0, sizeof tmp_buffer);
		
		tmp_buffer[1] = 0xFF;
			ATAPI_ClearBuf();
			atapi_buffer[0] = 0x42;				// Command code (Read SUBCHANNEL)
			atapi_buffer[1] = 0x02;
			atapi_buffer[2] = 0x40;
			atapi_buffer[3] = 0x01;				// 01 - current position
			atapi_buffer[7] = 0x08;				// 2 kB
			err = GetDataFromCD(tmp_buffer);	// Get data from CD
				_delay_ms(2);

		if(tmp_buffer[1] == 0x11)
			return 0;
		else
			return (tmp_buffer[1]);
}
//-----------------------------------------------------------------------------//
// Get Current Track Number
uint8_t  ATAPI_DA_GetCurTrack (void)
{
	return (tmp_buffer[6] - 1);
}
//-----------------------------------------------------------------------------//
// Get Current Time
uint16_t  ATAPI_DA_GetCurTime (void)
{
	return (convert_to_ms(tmp_buffer[13] , tmp_buffer[14]) );
}
//-----------------------------------------------------------------------------//
uint16_t GetSongTime(uint8_t songnum)
{
		uint16_t tmp1 = (60 * ATAPI_DA_GetM(songnum)) + ATAPI_DA_GetS(songnum);
		uint16_t tmp2 = (60 * ATAPI_DA_GetM(songnum+1)) + ATAPI_DA_GetS(songnum+1);
	return (tmp2-tmp1);
}
