#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<avr/pgmspace.h>

#include 	<stdio.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<stdlib.h>

#include	"ir.h"
#include 	"mydefs.h"

extern	uint8_t		state;
extern	uint8_t		inDex;

//-------------------------------------------------------------------------//
void init_ir (void)
{
 //Set Direction
 
    IR_DDR  &= ~IR_ICP; 
	TCCR1B  |= ( _BV(ICNC1) | _BV(CS12) | _BV(CS10));	// Time 1 input to clock/1024 input capture on falling edge  noise cancel
	TIMSK   |= _BV(TICIE1);   							// Enable input capture enable 
	IR_PORT	|= _BV(IR_ICP);	// pull up
}
//-------------------------------------------------------------------------//
uint16_t sony_ir_decode (uint16_t capture_counter)
{
	static 	uint16_t   	prev_capture_counter = 0;
	static 	uint8_t  	stop_bit = 0;
	static		uint16_t	ir_code = 0;
				uint16_t	width  = 0;	  

		if ( capture_counter > prev_capture_counter)
			width = capture_counter - prev_capture_counter;
		else
			width = capture_counter+(MAX_INT16 - prev_capture_counter);
//Check for start bit
		if ( width > IR_TV_PULSE_WIDTH)
		{
			if (width > IR_TV_START_BIT)  			//  Found Start bit 
			{
				stop_bit = 0;
				prev_capture_counter = 0;
				ir_code = 0;     			// New code
			}
			else 
			{	
				if (width > IR_TV_BIT1)	  		// bit 1 (3T)
				{
					ir_code <<= 1;    			//Shift right
					ir_code  |= 1;    			// Or 1
					stop_bit++;
				}
				else 	  // bit 0
				{
					ir_code  <<=  1;  //Shift right
					stop_bit++;
				}
			}
		}
		prev_capture_counter = capture_counter;
		if (stop_bit == IR_TV_STOP_BIT)
		{       //End code dispatch command
			stop_bit = 0;
			return ir_code;
		}   
		else
	return 0;		// No code detect yet
}
//-------------------------------------------------------------------------//
char remote_decode (uint16_t ircode)
{ 
  char nRet = 0;
  
	inDex = 0;
	
	switch (ircode)
	{
			case IR_TV_0:		state = 1 ;  inDex = 0; break;
			case IR_TV_1:		state = 1 ;  inDex = 1; break;
			case IR_TV_2:		state = 1 ;  inDex = 2; break;
			case IR_TV_3:		state = 1 ;  inDex = 3; break;
			case IR_TV_4:		state = 1 ;  inDex = 4; break;
			case IR_TV_5:		state = 1 ;  inDex = 5; break;
			case IR_TV_6:		state = 1 ;  inDex = 6; break;
			case IR_TV_7:		state = 1 ;  inDex = 7; break;
			case IR_TV_8:		state = 1 ;  inDex = 8; break;
			case IR_TV_9:		state = 1 ;  inDex = 9; break;
			case IR_TV_PULSE10:
								state = 10; 	break; //0x0A

			case IR_TV_PWR: 	state = 11; 	break;
			case IR_TV_PLAY:	state = 12; 	break;
			case IR_TV_PAUSE:	state = 13; 	break;						
			case IR_TV_FF:		state = 14; 	break;
			case IR_TV_REW:		state = 15; 	break;
			case IR_TV_STOP:	state = 16; 	break;
               			
			default: nRet = 1;
			          break;
 	}
	return nRet;
}
//-------------------------------------------------------------------------//
//-------------------------------------------------------------------------//
//-------------------------------------------------------------------------//
